/*
 * Copyright (c) 2008 Jesper Karlsson & David Gustafsson
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <gtk/gtk.h>
#include <string.h>
#include <libxfce4ui/libxfce4ui.h>

#include "xfbib-bibtex-string.h"
#include "xfbib-bibtex-value.h"

struct _XfbibBibtexString
{
	GObject parent;
	GObject *owner;
	gint internal;
	gchar separator;
	gchar *variable;
	XfbibBibtexValue *value;
};

typedef struct _XfbibBibtexStringClass
{
	GObjectClass parent;
} XfbibBibtexStringClass;

static void xfbib_bibtex_string_class_init(XfbibBibtexStringClass *klass);

static void xfbib_bibtex_string_init(XfbibBibtexString *instance);
static void xfbib_bibtex_string_finalize(GObject *obj);

G_DEFINE_TYPE(XfbibBibtexString, xfbib_bibtex_string, G_TYPE_OBJECT)

static void
xfbib_bibtex_string_class_init(XfbibBibtexStringClass *klass)
{
	GObjectClass *object_class = (GObjectClass *)klass;
	object_class->finalize = xfbib_bibtex_string_finalize;
}

static void
xfbib_bibtex_string_init(XfbibBibtexString *instance)
{
	instance->separator = '\0';
	instance->internal = 0;
	instance->variable = NULL;
	instance->value = xfbib_bibtex_value_new(G_OBJECT(instance));
}

static void
xfbib_bibtex_string_finalize(GObject *obj)
{
	G_OBJECT_CLASS(xfbib_bibtex_string_parent_class)->finalize(obj);
}

XfbibBibtexString *
xfbib_bibtex_string_new(GObject *obj)
{
	XfbibBibtexString *string;
	string = g_object_new(XFBIB_TYPE_BIBTEX_STRING, NULL);
	string->owner = obj;
	return string;
}


GObject *
xfbib_bibtex_string_get_owner(XfbibBibtexString *string)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_STRING(string), NULL);
	return string->owner;
}
	
const gchar *
xfbib_bibtex_string_get_variable(XfbibBibtexString *string)
{
	return string->variable;
}

void
xfbib_bibtex_string_set_variable(XfbibBibtexString *string, const gchar *variable)
{
	g_free(string->variable);
	string->variable = g_strdup(variable);
}

XfbibBibtexValue *
xfbib_bibtex_string_get_value(XfbibBibtexString *string)
{
	return string->value;
}

void
xfbib_bibtex_string_set_value(XfbibBibtexString *string, XfbibBibtexValue *value)
{
	string->value = value;
}

gboolean
xfbib_bibtex_string_parse(XfbibBibtexString *string, const gchar *str)
{
	gchar **split = NULL, *tmp;

	split = g_strsplit(str, "=", 2);

	if (split[1] == NULL) {
		/* Equal sign missing */
		g_warning(_("No equal sign found in definition of String\n"));
		g_strfreev(split);
		return FALSE;
	}

	g_free(string->variable);
	g_strstrip(split[0] + 7);

	/* Separator */
	string->separator = split[0][7];

	/* Variable */
	string->variable = g_strdup(split[0] + 8);

	/* Value */
	tmp = g_strndup(split[1], strlen(split[1]) - 1);
	if (!xfbib_bibtex_value_parse(string->value, tmp)) {
		g_free(tmp);
		g_strfreev(split);
		return FALSE;
	}
	g_free(tmp);

	g_strfreev(split);
	return TRUE;
}

gchar
xfbib_bibtex_string_get_separator(XfbibBibtexString *string)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_STRING(string), '\0');
	return string->separator;
}

gint
xfbib_bibtex_string_get_internal(XfbibBibtexString *string)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_STRING(string), -1);
	return string->internal;
}

void
xfbib_bibtex_string_set_internal(XfbibBibtexString *string, gint value)
{
	string->internal = value;
}
